@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2001-2007, International Business Machines
+*   Copyright (C) 2001-2008, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -27,6 +27,7 @@
 #include "unicode/uniset.h"
 
 #include "ucol_tok.h"
+#include "ucol_bld.h"
 #include "cmemory.h"
 #include "util.h"
 
@@ -1140,6 +1141,7 @@ static UColToken *ucol_tok_initAReset(UColTokenParser *src, UChar *expand, uint3
         // this is a syntax error
         *status = U_INVALID_FORMAT_ERROR;
         syntaxError(src->source,src->parsedToken.charsOffset-1,src->parsedToken.charsOffset+src->parsedToken.charsLen,parseError);
+        uprv_free(sourceToken);
         return 0;
     } else {
         sourceToken->prefix = 0;
@@ -1434,6 +1436,9 @@ uint32_t ucol_tok_assembleTokenList(UColTokenParser *src, UParseError *parseErro
                     // keep the flags around so that we know about before
                     sourceToken->flags = src->parsedToken.flags;
                     uhash_put(src->tailored, sourceToken, sourceToken, status);
+                    if(U_FAILURE(*status)) {
+                        return 0;
+                    }
                 } else {
                     /* we could have fished out a reset here */
                     if(sourceToken->strength != UCOL_TOK_RESET && lastToken != sourceToken) {
@@ -1746,7 +1751,7 @@ void ucol_tok_initTokenList(UColTokenParser *src, const UChar *rules, const uint
     // like copy and remove...
     //const UChar *openBrace = rules;
     int32_t optionNumber = -1;
-    const UChar *setStart;
+    const UChar *setStart = NULL;
     uint32_t i = 0;
     while(i < rulesLength) {
         if(rules[i] == 0x005B) {
@@ -1817,7 +1822,7 @@ void ucol_tok_initTokenList(UColTokenParser *src, const UChar *rules, const uint
     src->parsedToken.prefixOffset = 0;
     src->parsedToken.flags = 0;
     src->parsedToken.strength = UCOL_TOK_UNSET;
-
+    src->buildCCTabFlag = FALSE;
 
     if(U_FAILURE(*status)) {
         return;
