@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2009, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -8,6 +8,10 @@
 #ifndef _INTLTESTDECIMALFORMATAPI
 #define _INTLTESTDECIMALFORMATAPI
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "unicode/unistr.h"
 #include "intltest.h"
 
@@ -21,9 +25,12 @@ public:
      **/
     void testAPI(/*char *par*/);
     void testRounding(/*char *par*/);
+    void testRoundingInc(/*char *par*/);
 private:
     /*Helper functions */
     void verify(const UnicodeString& message, const UnicodeString& got, double expected);
 };
 
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
 #endif
