@@ -1,19 +1,21 @@
 /********************************************************************
- * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
- * others. All Rights Reserved.
- ********************************************************************/
-/*
-* File TMSGFMT.CPP
-*
-* Modification History:
-*
-*   Date        Name        Description
-*   03/24/97    helena      Converted from Java.
-*   07/11/97    helena      Updated to work on AIX.
-*   08/04/97    jfitz       Updated to intltest
-********************************************************************************
-*/
+ * Copyright (c) 1997-2009, International Business Machines
+ * Corporation and others. All Rights Reserved.
+ ********************************************************************
+ * File TMSGFMT.CPP
+ *
+ * Modification History:
+ *
+ *   Date        Name        Description
+ *   03/24/97    helena      Converted from Java.
+ *   07/11/97    helena      Updated to work on AIX.
+ *   08/04/97    jfitz       Updated to intltest
+ *******************************************************************/
+
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "tmsgfmt.h"
 
 #include "unicode/format.h"
@@ -23,8 +25,37 @@
 #include "unicode/numfmt.h"
 #include "unicode/choicfmt.h"
 #include "unicode/gregocal.h"
+#include <stdio.h>
 
-
+void
+TestMessageFormat::runIndexedTest(int32_t index, UBool exec,
+                                  const char* &name, char* /*par*/) {
+    switch (index) {
+        TESTCASE(0,testBug1);
+        TESTCASE(1,testBug2);
+        TESTCASE(2,sample);
+        TESTCASE(3,PatternTest);
+        TESTCASE(4,testStaticFormat);
+        TESTCASE(5,testSimpleFormat);
+        TESTCASE(6,testMsgFormatChoice);
+        TESTCASE(7,testCopyConstructor);
+        TESTCASE(8,testAssignment);
+        TESTCASE(9,testClone);
+        TESTCASE(10,testEquals);
+        TESTCASE(11,testNotEquals);
+        TESTCASE(12,testSetLocale);
+        TESTCASE(13,testFormat);
+        TESTCASE(14,testParse);
+        TESTCASE(15,testAdopt);
+        TESTCASE(16,testCopyConstructor2);
+        TESTCASE(17,TestUnlimitedArgsAndSubformats);
+        TESTCASE(18,TestRBNF);
+        TESTCASE(19,TestTurkishCasing);
+        TESTCASE(20,testAutoQuoteApostrophe);
+        TESTCASE(21,testMsgFormatPlural);
+        default: name = ""; break;
+    }
+}
 
 void TestMessageFormat::testBug3()
 {
@@ -100,9 +131,7 @@ void TestMessageFormat::testBug3()
         success = U_ZERO_ERROR;
         ParsePosition parsePos;
         form->parse(buffer, result, parsePos);
-#ifdef _DEBUG
-        it_out << " -> " /*<< dec*/ /*<< result*/ << "[supposed output for result]" <<endl;
-#endif
+        logln(UnicodeString(" -> ") /* + << dec*/ + toString(result) + UnicodeString("[supposed output for result]"));
         if (U_FAILURE(success)) {
             errln("Err: Number Format parse");
             logln("Number format parse failed.");
@@ -133,18 +162,16 @@ void TestMessageFormat::testBug2()
     UErrorCode status = U_ZERO_ERROR;
     UnicodeString result;
     // {sfb} use double format in pattern, so result will match (not strictly necessary)
-    const UnicodeString pattern = "There {0,choice,0.0#are no files|1.0#is one file|1.0<are {0, number} files} on disk {1}. ";
+    const UnicodeString pattern = "There {0,choice,0#are no files|1#is one file|1<are {0, number} files} on disk {1}. ";
     logln("The input pattern : " + pattern);
     MessageFormat *fmt = new MessageFormat(pattern, status);
     if (U_FAILURE(status)) {
         errln("MessageFormat pattern creation failed.");
-        logln("MessageFormat pattern creation failed.");
         return;
     }
     logln("The output pattern is : " + fmt->toPattern(result));
     if (pattern != result) {
         errln("MessageFormat::toPattern() failed.");
-        logln("MessageFormat::toPattern() failed.");
     }
     delete fmt;
 }
@@ -165,7 +192,6 @@ void TestMessageFormat::testBug2()
 
 #include "unicode/datefmt.h"
 #include <stdlib.h>
-#include <stdio.h>
 #include <string.h>
 
 IntlTest&
@@ -261,7 +287,7 @@ void TestMessageFormat::PatternTest()
         UnicodeString buffer;
         form = new MessageFormat(testCases[i], Locale::getUS(), success);
         if (U_FAILURE(success)) {
-            errln("MessageFormat creation failed.#1");
+            dataerrln("MessageFormat creation failed.#1 - %s", u_errorName(success));
             logln(((UnicodeString)"MessageFormat for ") + testCases[i] + " creation failed.\n");
             continue;
         }
@@ -279,7 +305,7 @@ void TestMessageFormat::PatternTest()
         FieldPosition fieldpos(0);
         form->format(testArgs, count, result, fieldpos, success);
         if (U_FAILURE(success)) {
-            errln("MessageFormat failed test #3");
+            dataerrln("MessageFormat failed test #3 - %s", u_errorName(success));
             logln("TestMessageFormat::PatternTest failed test #3");
             continue;
         }
@@ -337,46 +363,80 @@ void TestMessageFormat::sample()
     UnicodeString def("def");
     Formattable testArgs1[] = { abc, def };
     FieldPosition fieldpos(0);
-    logln(form->toPattern(buffer1) + "; " + form->format(testArgs1, 2, buffer2, fieldpos, success));
+    assertEquals("format",
+                 "There are abc files on def",
+                 form->format(testArgs1, 2, buffer2, fieldpos, success));
+    assertSuccess("format", success);
     delete form;
 }
 
-
-void TestMessageFormat::testStaticFormat(/* char* par */)
+void TestMessageFormat::testStaticFormat()
 {
-    logln("running TestMessageFormat::testStaticFormat");
-
     UErrorCode err = U_ZERO_ERROR;
-    GregorianCalendar cal(err);   
     Formattable arguments[] = {
         (int32_t)7,
         Formattable(UDate(8.71068e+011), Formattable::kIsDate),
         "a disturbance in the Force"
         };
-   
-        UnicodeString result;
-        result = MessageFormat::format(
-            "At {1,time} on {1,date}, there was {2} on planet {0,number,integer}.",
-            arguments,
-            3,
-            result,
-            err);
 
-        if (U_FAILURE(err)) {
-            errln("TestMessageFormat::testStaticFormat #1");
-            logln(UnicodeString("TestMessageFormat::testStaticFormat failed test #1 with error code ")+(int32_t)err);
-            return;
-        }
+    UnicodeString result;
+    result = MessageFormat::format(
+        "At {1,time} on {1,date}, there was {2} on planet {0,number,integer}.",
+        arguments,
+        3,
+        result,
+        err);
 
-        static const UnicodeString expected = 
-                "At 12:20:00 PM on Aug 8, 1997, there was a disturbance in the Force on planet 7.";
-        if (result != expected) {
-            errln("TestMessageFormat::testStaticFormat failed on test");
-            logln( UnicodeString("     Result: ") + result );
-            logln( UnicodeString("   Expected: ") + expected );
-        }
+    if (U_FAILURE(err)) {
+        dataerrln("TestMessageFormat::testStaticFormat #1 - %s", u_errorName(err));
+        logln(UnicodeString("TestMessageFormat::testStaticFormat failed test #1 with error code ")+(int32_t)err);
+        return;
+    }
+
+    const UnicodeString expected(
+            "At 12:20:00 PM on Aug 8, 1997, there was a disturbance in the Force on planet 7.", "");
+    if (result != expected) {
+        errln("TestMessageFormat::testStaticFormat failed on test");
+        logln( UnicodeString("     Result: ") + result );
+        logln( UnicodeString("   Expected: ") + expected );
+    }
 }
 
+/* When the default locale is tr, make sure that the pattern can still be parsed. */
+void TestMessageFormat::TestTurkishCasing()
+{
+    UErrorCode err = U_ZERO_ERROR;
+    Locale  saveDefaultLocale;
+    Locale::setDefault( Locale("tr"), err );
+
+    Formattable arguments[] = {
+        (int32_t)7,
+        Formattable(UDate(8.71068e+011), Formattable::kIsDate),
+        "a disturbance in the Force"
+        };
+
+    UnicodeString result;
+    result = MessageFormat::format(
+        "At {1,TIME} on {1,DATE,SHORT}, there was {2} on planet {0,NUMBER,INTEGER}.",
+        arguments,
+        3,
+        result,
+        err);
+
+    if (U_FAILURE(err)) {
+        dataerrln("TestTurkishCasing #1 with error code %s", u_errorName(err));
+        return;
+    }
+
+    const UnicodeString expected(
+            "At 12:20:00 on 08.08.1997, there was a disturbance in the Force on planet 7.", "");
+    if (result != expected) {
+        errln("TestTurkishCasing failed on test");
+        errln( UnicodeString("     Result: ") + result );
+        errln( UnicodeString("   Expected: ") + expected );
+    }
+    Locale::setDefault( saveDefaultLocale, err );
+}
 
 void TestMessageFormat::testSimpleFormat(/* char* par */)
 {
@@ -395,7 +455,7 @@ void TestMessageFormat::testSimpleFormat(/* char* par */)
     FieldPosition ignore(FieldPosition::DONT_CARE);
     form->format(testArgs1, 2, string, ignore, err);
     if (U_FAILURE(err) || string != "The disk \"MyDisk\" contains 0 file(s).") {
-        errln(UnicodeString("TestMessageFormat::testSimpleFormat failed on test #1"));
+        dataerrln(UnicodeString("TestMessageFormat::testSimpleFormat failed on test #1 - ") + u_errorName(err));
     }
  
     ignore.setField(FieldPosition::DONT_CARE);
@@ -403,14 +463,14 @@ void TestMessageFormat::testSimpleFormat(/* char* par */)
     form->format(testArgs2, 2, string, ignore, err);
     if (U_FAILURE(err) || string != "The disk \"MyDisk\" contains 1 file(s).") {
         logln(string);
-        errln(UnicodeString("TestMessageFormat::testSimpleFormat failed on test #2")+string);
+        dataerrln(UnicodeString("TestMessageFormat::testSimpleFormat failed on test #2")+string + " - " + u_errorName(err));
     }
  
     ignore.setField(FieldPosition::DONT_CARE);
     string.remove();
     form->format(testArgs3, 2, string, ignore, err);
     if (U_FAILURE(err) || string != "The disk \"MyDisk\" contains 12 file(s).") {
-        errln(UnicodeString("TestMessageFormat::testSimpleFormat failed on test #3")+string);
+        dataerrln(UnicodeString("TestMessageFormat::testSimpleFormat failed on test #3")+string + " - " + u_errorName(err));
     }
 
     delete form;
@@ -450,7 +510,7 @@ void TestMessageFormat::testMsgFormatChoice(/* char* par */)
     Formattable testArgs3[] = {(int32_t)1273, "MyDisk"};    
     form->format(testArgs3, 2, string, ignore, err);
     if (string != "The disk \"MyDisk\" contains 1,273 files.") {
-        errln("TestMessageFormat::testMsgFormatChoice failed on test #3");
+        dataerrln("TestMessageFormat::testMsgFormatChoice failed on test #3 - %s", u_errorName(err));
     }
 
     delete form;
@@ -458,13 +518,105 @@ void TestMessageFormat::testMsgFormatChoice(/* char* par */)
 }
 
 
+void TestMessageFormat::testMsgFormatPlural(/* char* par */)
+{
+    logln("running TestMessageFormat::testMsgFormatPlural");
+
+    UErrorCode err = U_ZERO_ERROR;
+    UnicodeString t1("{0, plural, one{C''est # fichier} other{Ce sont # fichiers}} dans la liste."); 
+    UnicodeString t2("{argument, plural, one{C''est # fichier} other {Ce sont # fichiers}} dans la liste.");
+    UnicodeString t3("There {0, plural, one{is # zavod}few{are {0, number,###.0} zavoda} other{are # zavodov}} in the directory.");
+    UnicodeString t4("There {argument, plural, one{is # zavod}few{are {argument, number,###.0} zavoda} other{are #zavodov}} in the directory.");
+    UnicodeString t5("{0, plural, one {{0, number,C''''est #,##0.0# fichier}} other {Ce sont # fichiers}} dans la liste.");
+    MessageFormat* mfNum = new MessageFormat(t1, Locale("fr"), err);
+    if (U_FAILURE(err)) {
+        dataerrln("TestMessageFormat::testMsgFormatPlural #1 - argumentIndex - %s", u_errorName(err));
+        logln(UnicodeString("TestMessageFormat::testMsgFormatPlural #1 with error code ")+(int32_t)err);
+        return;
+    }
+    Formattable testArgs1((int32_t)0);
+    FieldPosition ignore(FieldPosition::DONT_CARE);
+    UnicodeString numResult1;
+    mfNum->format(&testArgs1, 1, numResult1, ignore, err);
+   
+    MessageFormat* mfAlpha = new MessageFormat(t2, Locale("fr"), err);
+    UnicodeString argName[] = {UnicodeString("argument")};
+    UnicodeString argNameResult;
+    mfAlpha->format(argName, &testArgs1, 1, argNameResult, err);
+    if (U_FAILURE(err)) {
+        errln("TestMessageFormat::testMsgFormatPlural #1 - argumentName");
+        logln(UnicodeString("TestMessageFormat::testMsgFormatPlural #1 with error code ")+(int32_t)err);
+        delete mfNum;
+        return;
+    }
+    if ( numResult1 != argNameResult){
+        errln("TestMessageFormat::testMsgFormatPlural #1");
+        logln(UnicodeString("The results of argumentName and argumentIndex are not the same."));
+    }
+    if ( numResult1 != UnicodeString("C\'est 0 fichier dans la liste.")) {
+        errln("TestMessageFormat::testMsgFormatPlural #1");
+        logln(UnicodeString("The results of argumentName and argumentIndex are not the same."));
+    }
+    err = U_ZERO_ERROR;
+  
+    delete mfNum;
+    delete mfAlpha;
+
+    MessageFormat* mfNum2 = new MessageFormat(t3, Locale("ru"), err);
+    numResult1.remove();
+    Formattable testArgs2((int32_t)4);
+    mfNum2->format(&testArgs2, 1, numResult1, ignore, err);
+    MessageFormat* mfAlpha2 = new MessageFormat(t4, Locale("ru"), err);
+    argNameResult.remove();
+    mfAlpha2->format(argName, &testArgs2, 1, argNameResult, err);
+
+    if (U_FAILURE(err)) {
+        errln("TestMessageFormat::testMsgFormatPlural #2 - argumentName");
+        logln(UnicodeString("TestMessageFormat::testMsgFormatPlural #2 with error code ")+(int32_t)err);
+        delete mfNum2;
+        return;
+    }
+    if ( numResult1 != argNameResult){
+        errln("TestMessageFormat::testMsgFormatPlural #2");
+        logln(UnicodeString("The results of argumentName and argumentIndex are not the same."));
+    }
+    if ( numResult1 != UnicodeString("There are 4,0 zavoda in the directory.")) {
+        errln("TestMessageFormat::testMsgFormatPlural #2");
+        logln(UnicodeString("The results of argumentName and argumentIndex are not the same."));
+    }
+
+    delete mfNum2;
+    delete mfAlpha2;
+    
+    // nested formats
+    err = U_ZERO_ERROR;
+    MessageFormat* msgFmt = new MessageFormat(t5, Locale("fr"), err);
+    if (U_FAILURE(err)) {
+        errln("TestMessageFormat::test nested PluralFormat with argumentName");
+        logln(UnicodeString("TestMessageFormat::test nested PluralFormat with error code ")+(int32_t)err);
+        delete msgFmt;
+        return;
+    }
+    Formattable testArgs3((int32_t)0);
+    argNameResult.remove();
+    msgFmt->format(&testArgs3, 1, argNameResult, ignore, err);
+    if (U_FAILURE(err)) {
+        errln("TestMessageFormat::test nested PluralFormat with argumentName");
+    }
+    if ( argNameResult!= UnicodeString("C'est 0,0 fichier dans la liste.")) {
+        errln(UnicodeString("TestMessageFormat::test nested named PluralFormat."));
+        logln(UnicodeString("The unexpected nested named PluralFormat."));
+    }
+    delete msgFmt;
+}
+
+
 //---------------------------------
 //  API Tests
 //---------------------------------
 
 void TestMessageFormat::testCopyConstructor() 
 {
-    logln("TestMessageFormat::testCopyConstructor");
     UErrorCode success = U_ZERO_ERROR;
     MessageFormat *x = new MessageFormat("There are {0} files on {1}", success);
     MessageFormat *z = new MessageFormat("There are {0} files on {1} created", success);
@@ -495,7 +647,6 @@ void TestMessageFormat::testCopyConstructor()
 
 void TestMessageFormat::testAssignment() 
 {
-    logln("TestMessageFormat::testAssignment");
     UErrorCode success = U_ZERO_ERROR;
     MessageFormat *x = new MessageFormat("There are {0} files on {1}", success);
     MessageFormat *z = new MessageFormat("There are {0} files on {1} created", success);
@@ -525,7 +676,6 @@ void TestMessageFormat::testAssignment()
 
 void TestMessageFormat::testClone() 
 {
-    logln("TestMessageFormat::testClone");
     UErrorCode success = U_ZERO_ERROR;
     MessageFormat *x = new MessageFormat("There are {0} files on {1}", success);
     MessageFormat *z = new MessageFormat("There are {0} files on {1} created", success);
@@ -555,7 +705,6 @@ void TestMessageFormat::testClone()
 
 void TestMessageFormat::testEquals() 
 {
-    logln("TestMessageFormat::testClone");
     UErrorCode success = U_ZERO_ERROR;
     MessageFormat x("There are {0} files on {1}", success);
     MessageFormat y("There are {0} files on {1}", success);
@@ -580,7 +729,7 @@ void TestMessageFormat::testNotEquals()
     y.applyPattern("There are {0} files on {1} the disk", success);
     if (!(x != y)) {
         errln( "TestMessageFormat::testEquals failed #1");
-        logln("First test (operator !=): Failed!");
+        logln("Second test (operator !=): Failed!");
     }
 }
 
@@ -609,8 +758,10 @@ void TestMessageFormat::testSetLocale()
     // Just use unlocalized currency symbol.
     //UnicodeString compareStrGer = "At <time> on 08.08.1997, you made a deposit of 456,83 DM.";
     UnicodeString compareStrGer = "At <time> on 08.08.1997, you made a deposit of ";
+    compareStrGer += "456,83";
+    compareStrGer += (UChar) 0x00a0;
     compareStrGer += (UChar) 0x00a4;
-    compareStrGer += " 456,83.";
+    compareStrGer += ".";
 
     MessageFormat msg( formatStr, err);
     result = "";
@@ -624,19 +775,19 @@ void TestMessageFormat::testSetLocale()
 
     logln(result);
     if (result != compareStrEng) {
-        errln("***  MSG format err.");
+        dataerrln("***  MSG format err. - %s", u_errorName(err));
     }
 
     msg.setLocale(Locale::getEnglish());
     UBool getLocale_ok = TRUE;
-    if (msg.getLocale() != Locale::ENGLISH) {
+    if (msg.getLocale() != Locale::getEnglish()) {
         errln("*** MSG getLocal err.");
         getLocale_ok = FALSE;
     }
 
     msg.setLocale(Locale::getGerman());
 
-    if (msg.getLocale() != Locale::GERMAN) {
+    if (msg.getLocale() != Locale::getGerman()) {
         errln("*** MSG getLocal err.");
         getLocale_ok = FALSE;
     }
@@ -656,7 +807,7 @@ void TestMessageFormat::testSetLocale()
     if (result == compareStrGer) {
         logln("MSG setLocale tested.");
     }else{
-        errln( "*** MSG setLocale err.");
+        dataerrln( "*** MSG setLocale err. - %s", u_errorName(err));
     }
 
     if (getLocale_ok) { 
@@ -698,7 +849,7 @@ void TestMessageFormat::testFormat()
         err);
 
     if (err != U_ILLEGAL_ARGUMENT_ERROR) {
-        errln("*** MSG format without expected error code.");
+        dataerrln("*** MSG format without expected error code. - %s", u_errorName(err));
     }
     err = U_ZERO_ERROR;
 
@@ -714,7 +865,7 @@ void TestMessageFormat::testFormat()
     logln("MSG format( Formattable&, ... ) expected:" + compareStr);
     logln("MSG format( Formattable&, ... )   result:" + result);
     if (result != compareStr) {
-        errln("***  MSG format( Formattable&, .... ) err.");
+        dataerrln("***  MSG format( Formattable&, .... ) err. - %s", u_errorName(err));
     }else{
         logln("MSG format( Formattable&, ... ) tested.");
     }
@@ -821,7 +972,7 @@ void TestMessageFormat::testAdopt()
     Format** formatsToAdopt;
 
     if (!formats || !formatsCmp || (count <= 0) || (count != countCmp)) {
-        errln("Error getting Formats");
+        dataerrln("Error getting Formats");
         return;
     }
 
@@ -856,7 +1007,7 @@ void TestMessageFormat::testAdopt()
         b = formatsCmp[i];
         if ((a != NULL) && (b != NULL)) {
             if (*a == *b) {
-                logln("formatsChg != formatsCmp at index %d", i);
+                logln("formatsChg == formatsCmp at index %d", i);
                 diff = FALSE;
             }
         }
@@ -876,12 +1027,8 @@ void TestMessageFormat::testAdopt()
         return;
     }
 
-#if 1
-    msgCmp.toPattern( patCmp );
-    logln("MSG patCmp: " + patCmp);
-    msg.toPattern( patAct );
-    logln("MSG patAct: " + patAct);
-#endif
+    assertEquals("msgCmp.toPattern()", formatStr, msgCmp.toPattern(patCmp.remove()));
+    assertEquals("msg.toPattern()", formatStr, msg.toPattern(patAct.remove()));
 
     for (i = 0; i < countAct; i++) {
         a = formatsAct[i];
@@ -899,7 +1046,6 @@ void TestMessageFormat::testAdopt()
     }
     logln("MSG setFormats tested.");
 
-
     //----
 
     msg.applyPattern( formatStrChange, err ); //set msg formats to something different
@@ -924,12 +1070,8 @@ void TestMessageFormat::testAdopt()
     msg.adoptFormats( formatsToAdopt, countCmp ); // function to test
     delete[] formatsToAdopt;
 
-#if 1
-    msgCmp.toPattern( patCmp );
-    logln("MSG patCmp: " + patCmp);
-    msg.toPattern( patAct );
-    logln("MSG patAct: " + patAct);
-#endif
+    assertEquals("msgCmp.toPattern()", formatStr, msgCmp.toPattern(patCmp.remove()));
+    assertEquals("msg.toPattern()", formatStr, msg.toPattern(patAct.remove()));
 
     formatsAct = msg.getFormats(countAct);
     if (!formatsAct || (countAct <=0) || (countAct != countCmp)) {
@@ -979,12 +1121,8 @@ void TestMessageFormat::testAdopt()
     }
     delete[] formatsToAdopt; // array itself not needed in this case;
 
-#if 1
-    msgCmp.toPattern( patCmp );
-    logln("MSG patCmp: " + patCmp);
-    msg.toPattern( patAct );
-    logln("MSG patAct: " + patAct);
-#endif
+    assertEquals("msgCmp.toPattern()", formatStr, msgCmp.toPattern(patCmp.remove()));
+    assertEquals("msg.toPattern()", formatStr, msg.toPattern(patAct.remove()));
 
     formatsAct = msg.getFormats(countAct);
     if (!formatsAct || (countAct <=0) || (countAct != countCmp)) {
@@ -1014,7 +1152,7 @@ void TestMessageFormat::testAdopt()
 // which is probably why it didn't show up in the regular test for the copy constructor.)
 // For this reason, the test isn't changed even though it contains function calls whose results are
 // not tested and had no problems. Actually, the test failed by *crashing*.
-static void testCopyConstructor2()
+static void _testCopyConstructor2()
 {
     UErrorCode status = U_ZERO_ERROR;
     UnicodeString formatStr("Hello World on {0,date,full}", "");
@@ -1051,32 +1189,163 @@ static void testCopyConstructor2()
     delete fmt4;
 }
 
+void TestMessageFormat::testCopyConstructor2() {
+    _testCopyConstructor2();
+}
 
-void TestMessageFormat::runIndexedTest( int32_t index, UBool exec, const char* &name, char* /*par*/ )
-{
-    if (exec) logln("TestSuite MessageFormat");
+/**
+ * Verify that MessageFormat accomodates more than 10 arguments and
+ * more than 10 subformats.
+ */
+void TestMessageFormat::TestUnlimitedArgsAndSubformats() {
+    UErrorCode ec = U_ZERO_ERROR;
+    const UnicodeString pattern =
+        "On {0,date} (aka {0,date,short}, aka {0,date,long}) "
+        "at {0,time} (aka {0,time,short}, aka {0,time,long}) "
+        "there were {1,number} werjes "
+        "(a {3,number,percent} increase over {2,number}) "
+        "despite the {4}''s efforts "
+        "and to delight of {5}, {6}, {7}, {8}, {9}, and {10} {11}.";
+    MessageFormat msg(pattern, ec);
+    if (U_FAILURE(ec)) {
+        dataerrln("FAIL: constructor failed - %s", u_errorName(ec));
+        return;
+    }
 
-    switch (index) {
-        case 0:  name = "testBug1";             if (exec) testBug1(); break;
-        case 1:  name = "testBug2";             if (exec) testBug2(); break;
-        case 2:  name = "sample";               if (exec) sample(); break;
-        case 3:  name = "PatternTest";          if (exec) PatternTest(); break;
-        case 4:  name = "testStaticFormat";     if (exec) testStaticFormat(/* par */); break;
-        case 5:  name = "testSimpleFormat";     if (exec) testSimpleFormat(/* par */); break;
-        case 6:  name = "testMsgFormatChoice";  if (exec) testMsgFormatChoice(/* par */); break;
+    const Formattable ARGS[] = {
+        Formattable(UDate(1e13), Formattable::kIsDate),
+        Formattable((int32_t)1303),
+        Formattable((int32_t)1202),
+        Formattable(1303.0/1202 - 1),
+        Formattable("Glimmung"),
+        Formattable("the printers"),
+        Formattable("Nick"),
+        Formattable("his father"),
+        Formattable("his mother"),
+        Formattable("the spiddles"),
+        Formattable("of course"),
+        Formattable("Horace"),
+    };
+    const int32_t ARGS_LENGTH = sizeof(ARGS) / sizeof(ARGS[0]);
+    Formattable ARGS_OBJ(ARGS, ARGS_LENGTH);
 
-        case 7:  name = "testCopyConstructor";  if (exec) testCopyConstructor(); break;
-        case 8:  name = "testAssignment";       if (exec) testAssignment(); break;
-        case 9:  name = "testClone";            if (exec) testClone(); break;
-        case 10: name = "testEquals";           if (exec) testEquals(); break;
-        case 11: name = "testNotEquals";        if (exec) testNotEquals(); break;
-        case 12: name = "testSetLocale";        if (exec) testSetLocale(); break;
-        case 13: name = "testFormat";           if (exec) testFormat(); break;
-        case 14: name = "testParse";            if (exec) testParse(); break;
-        case 15: name = "testAdopt";            if (exec) testAdopt(); break;
-        case 16: name = "testCopyConstructor2"; if (exec) testCopyConstructor2(); break;
-
-
-        default: name = ""; break; //needed to end loop
+    UnicodeString expected =
+        "On Nov 20, 2286 (aka 11/20/86, aka November 20, 2286) "
+        "at 9:46:40 AM (aka 9:46 AM, aka 9:46:40 AM PST) "
+        "there were 1,303 werjes "
+        "(a 8% increase over 1,202) "
+        "despite the Glimmung's efforts "
+        "and to delight of the printers, Nick, his father, "
+        "his mother, the spiddles, and of course Horace.";
+    UnicodeString result;
+    msg.format(ARGS_OBJ, result, ec);
+    if (result == expected) {
+        logln(result);
+    } else {
+        errln((UnicodeString)"FAIL: Got " + result +
+              ", expected " + expected);
     }
 }
+
+// test RBNF extensions to message format
+void TestMessageFormat::TestRBNF(void) {
+    // WARNING: this depends on the RBNF formats for en_US
+    Locale locale("en", "US", "");
+
+    UErrorCode ec = U_ZERO_ERROR;
+
+    UnicodeString values[] = {
+        // decimal values do not format completely for ordinal or duration, and 
+        // do not always parse, so do not include them
+        "0", "1", "12", "100", "123", "1001", "123,456", "-17",
+    };
+    int32_t values_count = sizeof(values)/sizeof(values[0]);
+
+    UnicodeString formats[] = {
+        "There are {0,spellout} files to search.",
+        "There are {0,spellout,%simplified} files to search.",
+        "The bogus spellout {0,spellout,%BOGUS} files behaves like the default.",
+        "This is the {0,ordinal} file to search.", // TODO fix bug, ordinal does not parse
+        "Searching this file will take {0,duration} to complete.",
+        "Searching this file will take {0,duration,%with-words} to complete.",
+    };
+    int32_t formats_count = sizeof(formats)/sizeof(formats[0]);
+
+    Formattable args[1];
+
+    NumberFormat* numFmt = NumberFormat::createInstance(locale, ec);
+    if (U_FAILURE(ec)) {
+        dataerrln("Error calling NumberFormat::createInstance()");
+        return;
+    }
+
+    for (int i = 0; i < formats_count; ++i) {
+        MessageFormat* fmt = new MessageFormat(formats[i], locale, ec);
+        logln((UnicodeString)"Testing format pattern: '" + formats[i] + "'");
+
+        for (int j = 0; j < values_count; ++j) {
+            ec = U_ZERO_ERROR;
+            numFmt->parse(values[j], args[0], ec);
+            if (U_FAILURE(ec)) {
+                errln((UnicodeString)"Failed to parse test argument " + values[j]);
+            } else {
+                FieldPosition fp(0);
+                UnicodeString result;
+                fmt->format(args, 1, result, fp, ec);
+                logln((UnicodeString)"value: " + toString(args[0]) + " --> " + result + UnicodeString(" ec: ") + u_errorName(ec));
+               
+                if (i != 3) { // TODO: fix this, for now skip ordinal parsing (format string at index 3)
+                    int32_t count = 0;
+                    Formattable* parseResult = fmt->parse(result, count, ec);
+                    if (count != 1) {
+                        errln((UnicodeString)"parse returned " + count + " args");
+                    } else if (parseResult[0] != args[0]) {
+                        errln((UnicodeString)"parsed argument " + toString(parseResult[0]) + " != " + toString(args[0]));
+                    }
+                    delete []parseResult;
+                }
+            }
+        }
+        delete fmt;
+    }
+    delete numFmt;
+}
+
+void TestMessageFormat::testAutoQuoteApostrophe(void) {
+    const char* patterns[] = { // pattern, expected pattern
+        "'", "''",
+        "''", "''",
+        "'{", "'{'",
+        "' {", "'' {",
+        "'a", "''a",
+        "'{'a", "'{'a",
+        "'{a'", "'{a'",
+        "'{}", "'{}'",
+        "{'", "{'",
+        "{'a", "{'a",
+        "{'a{}'a}'a", "{'a{}'a}''a",
+        "'}'", "'}'",
+        "'} '{'}'", "'} '{'}''",
+        "'} {{{''", "'} {{{'''",
+    };
+    int32_t pattern_count = sizeof(patterns)/sizeof(patterns[0]);
+
+    for (int i = 0; i < pattern_count; i += 2) {
+        UErrorCode status = U_ZERO_ERROR;
+        UnicodeString result = MessageFormat::autoQuoteApostrophe(patterns[i], status);
+        UnicodeString target(patterns[i+1]);
+        if (target != result) {
+            const int BUF2_LEN = 64;
+            char buf[256];
+            char buf2[BUF2_LEN];
+            int32_t len = result.extract(0, result.length(), buf2, BUF2_LEN);
+            if (len >= BUF2_LEN) {
+                buf2[BUF2_LEN-1] = 0;
+            }
+            sprintf(buf, "[%2d] test \"%s\": target (\"%s\") != result (\"%s\")\n", i/2, patterns[i], patterns[i+1], buf2);
+            errln(buf);
+        }
+    }
+}
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
