@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 1999-2006, International Business Machines
+*   Copyright (C) 1999-2008, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -41,7 +41,7 @@ the udata API for loading ICU data. Especially, a UDataInfo structure
 precedes the actual data. It contains platform properties values and the
 file format version.
 
-The following is a description of format version 4 .
+The following is a description of format version 5 .
 
 The format changes between version 3 and 4 because the properties related to
 case mappings and bidi/shaping are pulled out into separate files
@@ -51,6 +51,10 @@ structures are omitted, rather than rearranging everything.
 
 For details see "Changes in format version 4" below.
 
+Format version 5 became necessary because the bit field for script codes
+overflowed. Several bit fields got rearranged, and three (Script, Block,
+Word_Break) got widened by one bit each.
+
 Data contents:
 
 The contents is a parsed, binary form of several Unicode character
@@ -219,6 +223,13 @@ Also, some of the previously used properties vector bits are reserved again.
 The indexes[] values for the omitted structures are still filled in
 (indicating zero-length arrays) so that the swapper code remains unchanged.
 
+--- Changes in format version 5 ---
+
+Rearranged bit fields in the second trie (AT) because the script code field
+overflowed. Old code would have seen nonsensically low values for new, higher
+script codes.
+Modified bit fields in icu/source/common/uprops.h
+
 ----------------------------------------------------------------------------- */
 
 /* UDataInfo cf. udata.h */
@@ -232,8 +243,8 @@ static UDataInfo dataInfo={
     0,
 
     { 0x55, 0x50, 0x72, 0x6f },                 /* dataFormat="UPro" */
-    { 4, 0, UTRIE_SHIFT, UTRIE_INDEX_SHIFT },   /* formatVersion */
-    { 4, 0, 1, 0 }                              /* dataVersion */
+    { 5, 0, UTRIE_SHIFT, UTRIE_INDEX_SHIFT },   /* formatVersion */
+    { 5, 1, 0, 0 }                              /* dataVersion */
 };
 
 static UNewTrie *pTrie=NULL;
@@ -419,6 +430,7 @@ generateData(const char *dataDir, UBool csource) {
     if(csource) {
         /* write .c file for hardcoded data */
         UTrie trie={ NULL };
+        UTrie2 *trie2;
         FILE *f;
 
         utrie_unserialize(&trie, trieBlock, trieSize, &errorCode);
@@ -427,7 +439,36 @@ generateData(const char *dataDir, UBool csource) {
                 stderr,
                 "genprops error: failed to utrie_unserialize(uprops.icu main trie) - %s\n",
                 u_errorName(errorCode));
-            return;
+            exit(errorCode);
+        }
+
+        /* use UTrie2 */
+        dataInfo.formatVersion[0]=6;
+        dataInfo.formatVersion[2]=0;
+        dataInfo.formatVersion[3]=0;
+        trie2=utrie2_fromUTrie(&trie, 0, &errorCode);
+        if(U_FAILURE(errorCode)) {
+            fprintf(
+                stderr,
+                "genprops error: utrie2_fromUTrie() failed - %s\n",
+                u_errorName(errorCode));
+            exit(errorCode);
+        }
+        {
+            /* delete lead surrogate code unit values */
+            UChar lead;
+            trie2=utrie2_cloneAsThawed(trie2, &errorCode);
+            for(lead=0xd800; lead<0xdc00; ++lead) {
+                utrie2_set32ForLeadSurrogateCodeUnit(trie2, lead, trie2->initialValue, &errorCode);
+            }
+            utrie2_freeze(trie2, UTRIE2_16_VALUE_BITS, &errorCode);
+            if(U_FAILURE(errorCode)) {
+                fprintf(
+                    stderr,
+                    "genprops error: deleting lead surrogate code unit values failed - %s\n",
+                    u_errorName(errorCode));
+                exit(errorCode);
+            }
         }
 
         f=usrc_create(dataDir, "uchar_props_data.c");
@@ -440,13 +481,13 @@ generateData(const char *dataDir, UBool csource) {
                 "static const UVersionInfo dataVersion={",
                 dataInfo.dataVersion, 8, 4,
                 "};\n\n");
-            usrc_writeUTrieArrays(f,
+            usrc_writeUTrie2Arrays(f,
                 "static const uint16_t propsTrie_index[%ld]={\n", NULL,
-                &trie,
+                trie2,
                 "\n};\n\n");
-            usrc_writeUTrieStruct(f,
-                "static const UTrie propsTrie={\n",
-                &trie, "propsTrie_index", NULL, NULL,
+            usrc_writeUTrie2Struct(f,
+                "static const UTrie2 propsTrie={\n",
+                trie2, "propsTrie_index", NULL,
                 "};\n\n");
 
             additionalPropsSize=writeAdditionalData(f, additionalProps, sizeof(additionalProps), indexes);
@@ -458,6 +499,7 @@ generateData(const char *dataDir, UBool csource) {
                 "};\n\n");
             fclose(f);
         }
+        utrie2_close(trie2);
     } else {
         /* write the data */
         pData=udata_create(dataDir, DATA_TYPE, DATA_NAME, &dataInfo,
