@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 1999-2007, International Business Machines
+*   Copyright (C) 1999-2009, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -28,6 +28,7 @@
 #include "cstring.h"
 #include "uoptions.h"
 #include "gentest.h"
+#include "toolutil.h"
 
 #define DATA_NAME "test"
 #define DATA_TYPE "icu"
@@ -110,7 +111,7 @@ createData(const char* outputDirectory, UErrorCode *errorCode) {
     pData=udata_create(outputDirectory, DATA_TYPE, DATA_NAME, &dataInfo,
                        U_COPYRIGHT_STRING, errorCode);
     if(U_FAILURE(*errorCode)) {
-        fprintf(stderr, "gentest: unable to create data memory, error %d\n", *errorCode);
+        fprintf(stderr, "gentest: unable to create data memory, %s\n", u_errorName(*errorCode));
         exit(*errorCode);
     }
 
@@ -142,6 +143,7 @@ outputJavaStuff(const char* progname, const char *outputDir) {
     int32_t i,t,count;
     char file[512];
     FILE *out;
+    int32_t year = getCurrentYear();
 
     uprv_strcpy(file,outputDir);
     if(*outputDir &&  /* don't put a trailing slash if outputDir is empty */ 
@@ -158,7 +160,7 @@ outputJavaStuff(const char* progname, const char *outputDir) {
         return 1;
     }
 
-    fprintf(out, "/** Copyright (C) 2007, International Business Machines Corporation and Others. All Rights Reserved. **/\n\n");
+    fprintf(out, "/** Copyright (C) %d, International Business Machines Corporation and Others. All Rights Reserved. **/\n\n", year);
     fprintf(out, "/* NOTE: this file is AUTOMATICALLY GENERATED by gentest. */\n\n");
     fprintf(out, "package com.ibm.icu.dev.test.util;\n\n");
     fprintf(out, "public class DebugUtilitiesData extends Object {\n");
@@ -196,6 +198,10 @@ outputJavaStuff(const char* progname, const char *outputDir) {
             switch(t) {
             case UDBG_UCalendarDateFields:
             case UDBG_UCalendarMonths:
+                /* Temporary workaround for IS_LEAP_MOTH #6051 */
+                if (t == UDBG_UCalendarDateFields && i == 22) {
+                    fprintf(out, "com.ibm.icu.util.ChineseCalendar.%s, /* %d */", udbg_enumName((UDebugEnumType)t,i), i);
+                } else
                 fprintf(out, "com.ibm.icu.util.Calendar.%s, /* %d */", udbg_enumName((UDebugEnumType)t,i), i);
                 break;
             case UDBG_UDebugEnumType:
